/*-
 * Copyright (c) 2001, 2003 Allan Saddi <allan@saddi.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY ALLAN SADDI AND HIS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL ALLAN SADDI OR HIS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: dlist.h 308 2003-02-20 10:39:37Z asaddi $
 */

#ifndef _APS_DLIST_H
#define _APS_DLIST_H

/* Structures */
#define DLIST(name, type) struct name { struct type *head, *dummy, *tail; }
#define DLIST_NODE(name, type) struct name { struct type *next, *prev; }

/* Internal macros */
#define DLIST_OFFSETOF(type, field) (char *) &((struct type *) 0)->field
#define DLIST_LIST_HEAD(listp, type, field) (struct type *) ((char *) &(listp)->head - DLIST_OFFSETOF(type, field))
#define DLIST_LIST_TAIL(listp, type, field) (struct type *) ((char *) &(listp)->dummy - DLIST_OFFSETOF(type, field))

/* Initialization */
#define DLIST_INIT(listp, type, field) do { \
  (listp)->head = DLIST_LIST_TAIL(listp, type, field); \
  (listp)->dummy = 0; \
  (listp)->tail = DLIST_LIST_HEAD(listp, type, field); \
} while (0)

/* Insertion macros */
#define DLIST_INSERT_BEFORE(beforep, nodep, field) do { \
  void **temp; \
  (nodep)->field.next = beforep; \
  (nodep)->field.prev = (beforep)->field.prev; \
  temp = (void **) &(beforep)->field.prev; \
  (beforep)->field.prev->field.next = nodep; \
  *temp = nodep; \
} while (0)

#define DLIST_INSERT_AFTER(afterp, nodep, field) do { \
  void **temp; \
  (nodep)->field.next = (afterp)->field.next; \
  (nodep)->field.prev = afterp; \
  temp = (void **) &(afterp)->field.next; \
  (afterp)->field.next->field.prev = nodep; \
  *temp = nodep; \
} while (0)

#define DLIST_ADD_HEAD(listp, nodep, field) DLIST_INSERT_BEFORE((listp)->head, nodep, field)

#define DLIST_ADD_TAIL(listp, nodep, field) DLIST_INSERT_AFTER((listp)->tail, nodep, field)

/* Removal */
#define DLIST_REMOVE(nodep, field) do { \
  (nodep)->field.prev->field.next = (nodep)->field.next; \
  (nodep)->field.next->field.prev = (nodep)->field.prev; \
} while (0)

/* Testing/expressions */
#define DLIST_EMPTY(listp, field) !((listp)->head->field.next)

#define DLIST_HEAD(listp, field) ((listp)->head->field.next ? (listp)->head : 0)

#define DLIST_TAIL(listp, field) ((listp)->tail->field.prev ? (listp)->tail : 0)

#define DLIST_NEXT(nodep, field) (nodep)->field.next
#define DLIST_PREV(nodep, field) (nodep)->field.prev

/* Iteration macros */
#define DLIST_FOREACH(listp, nodep, field) \
  for ((nodep) = (listp)->head; \
       (nodep)->field.next; \
       (nodep) = (nodep)->field.next)

#define DLIST_FOREACH_REVERSE(listp, nodep, field) \
  for ((nodep) = (listp)->tail; \
       (nodep)->field.prev; \
       (nodep) = (nodep)->field.prev)

#endif /* !_APS_DLIST_H */
