/*-
 * Copyright (c) 1999 Allan Saddi <allan@saddi.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: nfconfig.h 1792 2005-05-24 22:58:02Z asaddi $
 */

#ifndef NFCONFIG_H
#define NFCONFIG_H

#include <limits.h>

#include "dlist.h"
#include "newsfish.h"

#define SERVERCONF_FILE_NAME ".newsfish-servers"
#define RC_FILE_NAME ".newsfishrc"
#define NEWSRC_FILE_NAME ".newsfish-newsrc"

DLIST(_ServerList, Server);
DLIST(_NewsgroupList, Newsgroup);

extern struct _ServerList ServerList;
extern struct _NewsgroupList NewsgroupList, WildNewsgroupList;

int ReadServerConfig (const char *serverConf);
int ReadConfig (const char *conf);
int ReadNewsRC (const char *newsrc);

/* Add a server to our list. */
struct Server *AddServer (const char *server);
/* Find a particular server. */
struct Server *FindServer (const char *server);

/* Add a newsgroup to our list. */
struct Newsgroup *AddNewsgroup (const char *newsgroup);
/* Find a particular newsgroup in our list. */
struct Newsgroup *FindNewsgroup (const char *newsgroup);
/* Write out the newsRC file. */
int WriteNewsRC (const char *newsrc);

#endif /* NFCONFIG_H */
